﻿using System;
using System.Linq;
using TSLab.Script.Realtime;

namespace TSLab.Script.Handlers
{
    [HandlerCategory("Portfolio")]
    public class FreeMoney : IBar2ValueDoubleHandler
    {
        public double Execute(ISecurity source, int barNum)
        {
            if(source is ISecurityRt)
            {
                var srt = (ISecurityRt) source;
                return srt.CurrencyBalance;
            }
            double profit = source.InitDeposit;
            foreach (var pos in source.Positions.GetClosedOrActiveForBar(barNum))
            {
                if(pos.IsActiveForbar(barNum))
                {
                    profit -= pos.EntryPrice * pos.Shares;
                }
                else
                {
                    profit += pos.Profit();
                }
            }
            return profit;
        }
    }

    [HandlerCategory("Portfolio")]
    public class EstimatedMoney : IBar2ValueDoubleHandler
    {
        public double Execute(ISecurity source, int barNum)
        {
            if (source is ISecurityRt)
            {
                var srt = (ISecurityRt)source;
                return srt.EstimatedBalance;
            }
            double profit = source.InitDeposit + WholeTimeProfit.Profit(source, barNum);
            return profit;
        }
    }

    [HandlerCategory("Portfolio")]
    public class WholeTimeProfit : IBar2ValueDoubleHandler
    {
        public double Execute(ISecurity source, int barNum)
        {
            return Profit(source, barNum);
        }

        public static double Profit(ISecurity source, int barNum)
        {
            return source.Positions.GetClosedOrActiveForBar(barNum)
                .Sum(pos => pos.IsActiveForbar(barNum) ? pos.OpenProfit(barNum) : pos.Profit());
        }
    }

    [HandlerCategory("Portfolio")]
    public class WholeDayProfit : IBar2ValueDoubleHandler
    {
        public double Execute(ISecurity source, int barNum)
        {
            return CalcProfit(source, barNum, 1);
        }

        public static double CalcProfit(ISecurity source, int barNum, int days)
        {
            var dt = source.Bars[barNum].Date;
            var startDay = new DateTime(dt.Year, dt.Month, dt.Day);
            var endDay = startDay.AddDays(1);
            if (days > 1)
            {
                startDay = startDay.AddDays(1 - days);
            }
            return source.Positions.GetClosedOrActiveForBar(barNum)
                .Where(pos => pos.IsActive || (pos.ExitBar.Date >= startDay && pos.EntryBar.Date < endDay))
                .Sum(pos => pos.IsActiveForbar(barNum) ? pos.OpenProfit(barNum) : pos.Profit());
        }
    }

    [HandlerCategory("Portfolio")]
    public class NumDaysProfit : BasePeriodIndicatorHandler, IBar2ValueDoubleHandler
    {
        public double Execute(ISecurity source, int barNum)
        {
            return WholeDayProfit.CalcProfit(source, barNum, Period);
        }
    }

    [HandlerCategory("Portfolio")]
    public class NumMinutesProfit : BasePeriodIndicatorHandler, IBar2ValueDoubleHandler
    {
        public double Execute(ISecurity source, int barNum)
        {
            var endDay = source.Bars[barNum].Date;
            var startDay = endDay.AddDays(-Period);
            return source.Positions.GetClosedOrActiveForBar(barNum)
                .Where(pos => pos.IsActive || (pos.ExitBar.Date >= startDay && pos.EntryBar.Date < endDay))
                .Sum(pos => pos.IsActiveForbar(barNum) ? pos.OpenProfit(barNum) : pos.Profit());
        }
    }

    [HandlerCategory("Portfolio")]
    public class NumPositionsProfit : BasePeriodIndicatorHandler, IBar2ValueDoubleHandler
    {
        public double Execute(ISecurity source, int barNum)
        {
            double profit = 0;
            double trades = Period;
            foreach (var pos in source.Positions
                                    .Where(pos => !pos.IsActiveForbar(barNum))
                                    .OrderByDescending(pos => pos.ExitBarNum))
            {
                profit += pos.OpenProfit(barNum);
                if (--trades <= 0)
                {
                    break;
                }
            }
            return profit;
        }
    }
}
