﻿using System;
using System.Linq;
using TSLab.DataSource;
using TSLab.Script.Realtime;

// ReSharper disable UnusedAutoPropertyAccessor.Global
// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable UnusedMember.Global
namespace TSLab.Script.Handlers
{
    [HandlerName("Shares")]
    [HandlerCategory("Position")]
    public class PositionShares : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null || pos.EntryBarNum > barNum)
            {
                return 0;
            }
            return pos.Shares;
        }
    }

    [HandlerName("Entry Price")]
    [HandlerName("Цена Входа", Language = "ru-RU")]
    [HandlerCategory("Position")]
    public class EntryPrice : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null || pos.EntryBarNum > barNum)
            {
                return 0;
            }
            return pos.EntryPrice;
        }
    }

    [HandlerName("Entry Price (Calculated)")]
    [HandlerName("Цена Входа (Расчетная)", Language = "ru-RU")]
    [HandlerCategory("Position")]
    public class CalcEntryPrice : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            return EntryPrice(pos, barNum);
        }

        public static double EntryPrice(IPosition pos, int barNum)
        {
            if (pos == null || pos.EntryBarNum > barNum)
            {
                return 0;
            }
            var posRt = pos as IPositionRt;
            if (posRt != null && posRt.EntryOrders.Count > 0)
            {
                var order = posRt.EntryOrders[0];
// ReSharper disable CompareOfFloatsByEqualityOperator
                if (order.OrderType == OrderType.Market || order.OrderPrice == 0)
// ReSharper restore CompareOfFloatsByEqualityOperator
                {
                    return pos.EntryBar.Open;
                }
                return order.OrderPrice;
            }
            return pos.EntryPrice;
        }
    }

    [HandlerName("Entry Date")]
    [HandlerCategory("Position")]
    public class EntryDate : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null || pos.EntryBarNum > barNum)
            {
                return 0;
            }
            var dt = pos.EntryBar.Date;
            return (dt.Year % 100) * 10000.0 + dt.Month * 100.0 + dt.Day;
        }
    }

    [HandlerName("Entry Time")]
    [HandlerCategory("Position")]
    public class EntryTime : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null || pos.EntryBarNum > barNum)
            {
                return 0;
            }
            var dt = pos.EntryBar.Date;
            return dt.Hour * 10000.0 + dt.Minute * 100.0 + dt.Second;
        }
    }

    [HandlerName("Bars Held")]
    [HandlerCategory("Position")]
    public class BarsHeld : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if(pos == null || pos.EntryBarNum >= barNum)
            {
                return 0;
            }
            if (pos.IsActive || pos.ExitBarNum > barNum)
            {
                return barNum - pos.EntryBarNum;
            }
            return pos.ExitBarNum - pos.EntryBarNum;
        }
    }

    [HandlerCategory("Position")]
    public class Profit : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null)
            {
                return 0;
            }
            return pos.OpenProfit(barNum);
        }
    }

    [HandlerName("Profit %")]
    [HandlerCategory("Position")]
    public class ProfitPct : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null)
            {
                return 0;
            }
// ReSharper disable CompareOfFloatsByEqualityOperator
            return pos.Shares == 0 ? 0 : pos.OpenProfitPct(barNum);
// ReSharper restore CompareOfFloatsByEqualityOperator
        }
    }

    [HandlerCategory("Position")]
    public class MAE : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null)
            {
                return 0;
            }
            return pos.OpenMAE(barNum);
        }
    }

    [HandlerName("MAE %")]
    [HandlerCategory("Position")]
    public class MAEPct : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null)
            {
                return 0;
            }
            return pos.OpenMAEPct(barNum);
        }
    }

    [HandlerCategory("Position")]
    public class MFE : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null)
            {
                return 0;
            }
            return pos.OpenMFE(barNum);
        }
    }

    [HandlerName("MFE %")]
    [HandlerCategory("Position")]
    public class MFEPct : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null)
            {
                return 0;
            }
            return pos.OpenMFEPct(barNum);
        }
    }

    [HandlerName("Trail Stop")]
    [HandlerCategory("Position")]
    public class TrailStop : IPosition2Double
    {
        [HandlerParameter(true, "1.5", Min = "0.1", Max = "5", Step = "0.1", Name = "Stop Loss")]
        public double StopLoss { get; set; }

        [HandlerParameter(true, "0.5", Min = "0.1", Max = "3", Step = "0.1", Name = "Trail Enable")]
        public double TrailEnable { get; set; }

        [HandlerParameter(true, "0.5", Min = "0.1", Max = "3", Step = "0.1", Name = "Trail Loss")]
        public double TrailLoss { get; set; }

        [HandlerParameter(true, "false", NotOptimized = true, Name = "Use Calc Price")]
        public bool UseCalcPrice { get; set; }

        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null)
            {
                return 0;
            }
            double stop;
            var curProfit = pos.OpenMFE(barNum);
            var entryPrice = pos.EntryPrice;
            if (UseCalcPrice)
            {
                var ep2 = CalcEntryPrice.EntryPrice(pos, barNum);
                var diff = (entryPrice - ep2) * (pos.IsLong ? 1 : -1);
                curProfit += diff;
                entryPrice = ep2;
            }
            curProfit *= 100/entryPrice*pos.Security.Margin;
            
            if (curProfit > TrailEnable)
            {
                double shift = (curProfit - TrailLoss) / 100;
                stop = entryPrice * (1 + (pos.IsLong ? shift : -shift));
            }
            else
            {
                double shift = (0 - StopLoss) / 100;
                stop = entryPrice * (1 + (pos.IsLong ? shift : -shift));
            }
            var lastStop = pos.GetStop(barNum);
// ReSharper disable CompareOfFloatsByEqualityOperator
            if(lastStop == 0)
// ReSharper restore CompareOfFloatsByEqualityOperator
            {
                return stop;
            }
            return pos.IsLong ? Math.Max(stop, lastStop) : Math.Min(stop, lastStop);
        }
    }

    [HandlerName("Trail Stop Abs")]
    [HandlerCategory("Position")]
    public class TrailStopAbs : IPosition2Double
    {
        [HandlerParameter(true, "150", Min = "10", Max = "500", Step = "5", Name = "Stop Loss")]
        public double StopLoss { get; set; }

        [HandlerParameter(true, "50", Min = "10", Max = "500", Step = "5", Name = "Trail Enable")]
        public double TrailEnable { get; set; }

        [HandlerParameter(true, "50", Min = "10", Max = "500", Step = "5", Name = "Trail Loss")]
        public double TrailLoss { get; set; }

        [HandlerParameter(true, "false", NotOptimized = true, Name = "Use Calc Price")]
        public bool UseCalcPrice { get; set; }

        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null)
            {
                return 0;
            }
            double stop;
            var curProfit = pos.OpenMFE(barNum);
            if (UseCalcPrice)
            {
                var ep1 = pos.EntryPrice;
                var ep2 = CalcEntryPrice.EntryPrice(pos, barNum);
                var diff = (ep1 - ep2) * (pos.IsLong ? 1 : -1);
                curProfit += diff;
            }
            if (curProfit > TrailEnable)
            {
                double shift = curProfit - TrailLoss;
                stop = pos.EntryPrice + (pos.IsLong ? shift : -shift);
            }
            else
            {
                double shift = -StopLoss;
                stop = pos.EntryPrice + (pos.IsLong ? shift : -shift);
            }
            var lastStop = pos.GetStop(barNum);
// ReSharper disable CompareOfFloatsByEqualityOperator
            if (lastStop == 0)
// ReSharper restore CompareOfFloatsByEqualityOperator
            {
                return stop;
            }
            return pos.IsLong ? Math.Max(stop, lastStop) : Math.Min(stop, lastStop);
        }
    }

    [HandlerName("Has Position Active")]
    [HandlerCategory("Position")]
    public class HasPositionActive : IBar2BoolHandler
    {
        public bool Execute(ISecurity source, int barNum)
        {
            if(source is ISecurityRt)
            {
                return Internal(source, barNum);
            }
            var pos = source.Positions.LastPositionActive;
            if(pos != null && pos.EntryBarNum > barNum)
            {
                return Internal(source, barNum);
            }
            var isActive = pos != null && pos.EntryBarNum <= barNum;
            pos = source.Positions.LastLongPositionClosed;
            var isLongActive =  !(pos == null || pos.ExitBarNum <= barNum);
            pos = source.Positions.LastShortPositionClosed;
            var isShortActive = !(pos == null || pos.ExitBarNum <= barNum);
            return isActive || isLongActive || isShortActive;
        }

        private static bool Internal(ISecurity source, int barNum)
        {
            foreach (var ps in source.Positions)
            {
                if (ps.EntryBarNum <= barNum)
                {
                    if (ps.IsActive)
                    {
                        return true;
                    }
                    if (ps.ExitBarNum > barNum)
                    {
                        return true;
                    }
                }
            }
            return false;
        }
    }

    [HandlerName("Has Long Position Active")]
    [HandlerCategory("Position")]
    public class HasLongPositionActive : IBar2BoolHandler
    {
        public bool Execute(ISecurity source, int barNum)
        {
            if (source is ISecurityRt)
            {
                return Internal(source, barNum);
            }
            var pos = source.Positions.LastLongPositionActive;
            if (pos != null && pos.EntryBarNum > barNum)
            {
                return Internal(source, barNum);
            }
            var isActive = pos != null && pos.EntryBarNum <= barNum;
            pos = source.Positions.LastLongPositionClosed;
            return isActive || !(pos == null || pos.ExitBarNum <= barNum);
        }

        private static bool Internal(ISecurity source, int barNum)
        {
            foreach (var ps in source.Positions)
            {
                if(ps.IsShort)
                {
                    continue;
                }
                if (ps.EntryBarNum <= barNum)
                {
                    if (ps.IsActive)
                    {
                        return true;
                    }
                    if (ps.ExitBarNum > barNum)
                    {
                        return true;
                    }
                }
            }
            return false;
        }
    }

    [HandlerName("Has Short Position Active")]
    [HandlerCategory("Position")]
    public class HasShortPositionActive : IBar2BoolHandler
    {
        public bool Execute(ISecurity source, int barNum)
        {
            if (source is ISecurityRt)
            {
                return Internal(source, barNum);
            }
            var pos = source.Positions.LastShortPositionActive;
            if (pos != null && pos.EntryBarNum > barNum)
            {
                return Internal(source, barNum);
            }
            var isActive = pos != null && pos.EntryBarNum <= barNum;
            pos = source.Positions.LastShortPositionClosed;
            return isActive || !(pos == null || pos.ExitBarNum <= barNum);
        }

        private static bool Internal(ISecurity source, int barNum)
        {
            foreach (var ps in source.Positions)
            {
                if (ps.IsLong)
                {
                    continue;
                }
                if (ps.EntryBarNum <= barNum)
                {
                    if (ps.IsActive)
                    {
                        return true;
                    }
                    if (ps.ExitBarNum > barNum)
                    {
                        return true;
                    }
                }
            }
            return false;
        }
    }

    [HandlerName("Last is Closed and was Long")]
    [HandlerCategory("Position")]
    public class LastClosedIsLong : IBar2BoolHandler
    {
        public bool Execute(ISecurity source, int barNum)
        {
            if(source.Positions.Count() == 0)
            {
                return false;
            }
            var last = source.Positions.Last();
            return !last.IsActive && last.IsLong && last.ExitBarNum <= barNum;
        }
    }

    [HandlerName("Last is Closed and was Short")]
    [HandlerCategory("Position")]
    public class LastClosedIsShort : IBar2BoolHandler
    {
        public bool Execute(ISecurity source, int barNum)
        {
            if (source.Positions.Count() == 0)
            {
                return false;
            }
            var last = source.Positions.Last();
            return !last.IsActive && last.IsShort && last.ExitBarNum <= barNum;
        }
    }

    [HandlerName("Last Closed Is Long")]
    [HandlerCategory("Position")]
    public class LastClosedIsLong2 : IBar2BoolHandler
    {
        public bool Execute(ISecurity source, int barNum)
        {
            var last = source.Positions.LastPositionClosed;
            if (last == null)
            {
                return false;
            }
            return last.IsLong && last.ExitBarNum <= barNum;
        }
    }


    [HandlerName("Last Closed Is Short")]
    [HandlerCategory("Position")]
    public class LastClosedIsShort2 : IBar2BoolHandler
    {
        public bool Execute(ISecurity source, int barNum)
        {
            var last = source.Positions.LastPositionClosed;
            if (last == null)
            {
                return false;
            }
            return last.IsShort && last.ExitBarNum <= barNum;
        }
    }
}
// ReSharper restore MemberCanBePrivate.Global
// ReSharper restore UnusedAutoPropertyAccessor.Global
// ReSharper restore UnusedMember.Global
