﻿using System;
using System.Linq;

namespace TSLab.Script.Handlers
{
    [HandlerCategory("Position")]
    public class DrowdownCount : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        public double Execute(ISecurity source, int barNum)
        {
            return (from pos in source.Positions.OrderByDescending(pos => pos.ExitBarNum)
                    where !pos.IsActive
                    select pos.Profit()).TakeWhile(profit => profit <= 0).Count();
        }
    }

    [HandlerCategory("Position")]
    public class HasTwoLoss : IOneSourceHandler, IBooleanReturns, IValuesHandler, ISecurityInputs
    {
        public bool Execute(ISecurity source, int barNum)
        {
            var list = source.Positions.Where(pos => !pos.IsActive && pos.EntryBarNum < barNum).OrderByDescending(pos => pos.ExitBar.Date).ToArray();
            if (list.Length < 2)
            {
                return false;
            }
            return list[0].Profit() < 0 && list[1].Profit() < 0;
        }
    }

    [HandlerCategory("Position")]
    [HandlerName("Last Exit has this name")]
    public class PosActiveNameExit : IBar2BoolHandler
    {
        [HandlerParameter(true, "", NotOptimized = true)]
        public string Name { get; set; }

        #region IBar2BoolHandler Members

        public bool Execute(ISecurity source, int barNum)
        {
            IPosition pos = source.Positions.LastPosition;
            string name = pos == null || !(!pos.IsActive && pos.ExitBarNum <= barNum) ? null : pos.ExitSignalName;
            return name == Name;
        }

        #endregion
    }

    [HandlerCategory("Position")]
    [HandlerName("Last Closed Position Time")]
    public class LastClosedPositionTime : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        public double Execute(ISecurity source, int barNum)
        {
            var pos = source.Positions.LastPositionClosed;
            if(pos != null && pos.ExitBarNum > barNum)
            {
                pos = null;
            }
            var dt = pos == null ? DateTime.MinValue : pos.EntryBar.Date;
            return dt.Hour*10000.0 + dt.Minute*100.0 + dt.Second;
        }
    }

    [HandlerCategory("Position")]
    [HandlerName("Last Closed Position Date")]
    public class LastClosedPositionDate : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        public double Execute(ISecurity source, int barNum)
        {
            var pos = source.Positions.LastPositionClosed;
            if (pos != null && pos.ExitBarNum > barNum)
            {
                pos = null;
            }
            var dt = pos == null ? DateTime.MinValue : pos.EntryBar.Date;
            return (dt.Year % 100) * 10000.0 + dt.Month * 100.0 + dt.Day;
        }
    }

    [HandlerCategory("Position")]
    [HandlerName("Last Closed Position Exit Time")]
    public class LastClosedPositionExitTime : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        public double Execute(ISecurity source, int barNum)
        {
            var pos = source.Positions.LastPositionClosed;
            if (pos != null && pos.ExitBarNum > barNum)
            {
                pos = null;
            }
            var dt = pos == null ? DateTime.MinValue : pos.ExitBar.Date;
            return dt.Hour * 10000.0 + dt.Minute * 100.0 + dt.Second;
        }
    }

    [HandlerCategory("Position")]
    [HandlerName("Last Closed Position Exit Date")]
    public class LastClosedPositionExitDate : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        public double Execute(ISecurity source, int barNum)
        {
            var pos = source.Positions.LastPositionClosed;
            if (pos != null && pos.ExitBarNum > barNum)
            {
                pos = null;
            }
            var dt = pos == null ? DateTime.MinValue : pos.ExitBar.Date;
            return (dt.Year % 100) * 10000.0 + dt.Month * 100.0 + dt.Day;
        }
    }

    [HandlerCategory("Position")]
    [HandlerName("Last Closed Named Position Exit Time")]
    public class LastClosedNamePositionExitTime : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        [HandlerParameter(true, "", NotOptimized = true)]
        public string Name { get; set; }

        public double Execute(ISecurity source, int barNum)
        {
            var pos = source.Positions.GetLastForSignal(Name);
            if (pos != null && (pos.IsActive || pos.ExitBarNum > barNum))
            {
                pos = null;
            }
            var dt = pos == null ? DateTime.MinValue : pos.ExitBar.Date;
            return dt.Hour * 10000.0 + dt.Minute * 100.0 + dt.Second;
        }
    }

    [HandlerCategory("Position")]
    [HandlerName("Last Closed Named Position Exit Date")]
    public class LastClosedNamePositionExitDate : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        [HandlerParameter(true, "", NotOptimized = true)]
        public string Name { get; set; }

        public double Execute(ISecurity source, int barNum)
        {
            var pos = source.Positions.GetLastForSignal(Name);
            if (pos != null && (pos.IsActive || pos.ExitBarNum > barNum))
            {
                pos = null;
            }
            var dt = pos == null ? DateTime.MinValue : pos.ExitBar.Date;
            return (dt.Year % 100) * 10000.0 + dt.Month * 100.0 + dt.Day;
        }
    }

    [HandlerCategory("Position")]
    [HandlerName("Days In Position")]
    public class DaysInPosition : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null)
            {
                return 0;
            }
            var dt = pos.EntryBar.Date;
            var curDt = pos.Security.Bars[barNum].Date;
            return (curDt-dt).TotalDays;
        }
    }

    [HandlerCategory("Position")]
    [HandlerName("Minutes In Position")]
    public class MinutesInPosition : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null)
            {
                return 0;
            }
            var dt = pos.EntryBar.Date;
            var curDt = pos.Security.Bars[barNum].Date;
            return (curDt - dt).TotalMinutes;
        }
    }

    [HandlerCategory("Position")]
    [HandlerName("Last Exit Price")]
    public class LastExitPrice : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        public double Execute(ISecurity source, int barNum)
        {
            var pos = source.Positions.LastPositionClosed;
            if (pos != null && pos.ExitBarNum > barNum)
            {
                pos = null;
            }
            return pos == null ? 0 : pos.ExitPrice;
        }
    }
}
